function [L E exitFlag ite] = Block_PCA_admm(D,W,lambda1,lambda2,varargin)
% using ADMM (splitting method)  to solve Block PCA
% Inputs:
%  D: the data matrix
%  W: the weighted matrix, same size as D, with all elements nonnegative
%  lambda1,lambda2: positive scalar
%  arg: Optional, a structure with the following optional items:
%    '.debug_mode'  1: put out iteration information,  0: do not put out;
%    '.ite_max'     maximum iteration number;
%    '.epsilon_1'   
%    '.epsilon_2'   algorithm terminates if |r|< epsilon_1 &&
%                     |s|<epsilon_2, with r, s the residual and dual residual,
%                     resp., set this two arguments at the same time, if
%                     necessary.
%    '.beta'        penalty parameter, see [1] for details. 
%    '.mu'          penalty parameter, see [1] for details. Be cautious to
%                     set this two argument (beta and mu), since it greatly
%                     effects the convergence speeds; (mu should greater
%                     than 2, see [1]).
%  ite: iteration number
% Outputs:
%  L: the block low rank matrix;
%  E: the matrix consisting errors/noise;
%  exitFlag: 1: terminate within the given precision; 0: exceed the maximum
%    iteration information;

%    Block PCA:
%    min_{L,E} |L|_* + lambda1 * \sum_{i,j} w_{i,j}|L_{i,j}| + lambda2*|E|_1
%    s.t. L + E = D

% For the splitting method, refer
% [1] B.S. He, M. Tao, and X.M. Yuan, A splitting method for separate convex 
%     programming with linking linear constraints.
%     http://www.optimization-online.org/DB\_HTML/2010/06/2665.html

%  The Block PCA can be recast as 
%    min_{L,E} |A|_* + lambda1 * \sum_{i,j} w_{i,j}|U_{i,j}| + lambda2*|E|_1
%    s.t.  [ D   =  [ L   + [ E    -  [ 0
%            0]       L]      0]        U]


% Arguments.
if nargin>4 
    arg = varargin{1};
else
    arg = [];
end

if isstruct(arg) && isfield(arg,'debug_mode')
    debug_mode = arg.debug_mode;
else
    debug_mode = 0; 
end

if isstruct(arg) && isfield(arg,'beta')
    beta = arg.beta;
else
    beta = 10/norm(D,'fro');
end

if isstruct(arg) && isfield(arg,'mu')
    mu = arg.mu;
else    
    mu = 10; % (mu>m-1 = 3-1 =2 by [1])
end

if isstruct(arg) && isfield(arg,'ite_max')
    ite_max = arg.ite_max;
else
    ite_max = 8000; % maximum iteration number
end

if isstruct(arg) && isfield(arg,'epsilon_1') && isfield(arg,'epsilon_2')
    epsilon_1 = arg.epsilon_1;
    epsilon_2 = arg.epsilon_2;
else
    epsilon_1 = 1.0E-6; 
    epsilon_2 = 1.0E-6;
end
% bound_r_s = 10; % leading that:  |r|/|s|<bound_r_s and |s|/|r|<bound_r_s
% delta_desc = 2;
% delta_incr = 2; 

% Initialization.
[m n] = size(D);
L = D;
E = zeros(m,n);
U = D;
LLambda1 = zeros(m,n);
LLambda2 = zeros(m,n);

% Iterations.
normF_D = norm(D,'fro');
ite = 1;
while 1    
    tau = 0.5/beta;
    tau_1 = 1/(beta*mu);
    tau_e = lambda2*tau_1;
    W1 = lambda1*tau_1*W;
    % 1) solve L1.
    Y = 0.5*(D-E+U)+tau*(LLambda1+LLambda2);    
    L1 =  singular_value_threshold(tau,Y);
    % 2) pdate the Lagrangian multipliers LLambda1_half and LLambda2_half
    LLambda1_half = LLambda1 - beta*(L1+E-D);
    LLambda2_half = LLambda2 - beta*(L1-U);
    % 3) solve U1, E1.
    U1 = threshold_l1(W1, U-tau_1*LLambda2_half);
    E1 = threshold_l1(tau_e,E+tau_1*LLambda1_half);
    % 4) update the Lagrangian multipliers LLambda1 and LLambda2
    LLambda1 = LLambda1 - beta*(L1 + E1 -D); % Note that on the rightside of the equation, 
                                    %   it is LLambda1, not LLambda1_half
    LLambda2 = LLambda2 - beta*(L1 -U1);
    % stop criteria
    normF_r = sqrt(norm(L1+E1-D,'fro').^2 + norm(L1-U1,'fro').^2);
    normF_s = sqrt(norm(L1-L,'fro').^2 + norm(E1-E,'fro').^2 +norm(U1-U,'fro').^2);
  
    if normF_r/normF_D <= epsilon_1 && beta*normF_s/normF_D <= epsilon_2 
       exitFlag = 1;
       break;
    end
    
    if debug_mode==1 && mod(ite,50)==1
        if ite==1
            fprintf('|ite \t| beta\t\t | norm_r (norm_LED, norm_LU)\t | norm_s (norm_L,norm_E,norm_U) \t|\n');
        else  
            fprintf('|%d  \t| %.2e\t | %.2e  (%.1e, %.1e)\t |%.2e  (%.1e, %.1e, %.1e)\t|\n',...
                ite,beta,normF_r,norm(L1+E1-D,'fro'),norm(L1-U1,'fro'),...
                normF_s,norm(L1-L,'fro'),norm(E1-E,'fro'),norm(U1-U,'fro'));
        end
    end
 
    if ite>ite_max
        fprintf(1,'Block_PCA_admm exceeds the maximum iteration number: %d.\n',ite_max);
        exitFlag = 0;
        break;
    end
        % update beta
%      if normF_r/(beta*normF_s) > bound_r_s
%          beta = beta*delta_incr;
%      elseif (beta*normF_s)/normF_r > bound_r_s
%          beta = beta/delta_desc;
%      end
    % update ite, L, E, U
    ite = ite + 1;
    L = L1;
    E = E1; 
    U = U1;      
end
end

function Y1 = singular_value_threshold(tau,Y)
% do singular value thresholding on Y
[U S V] = svd(Y,'econ');
s = diag(S);
s = s-tau;
num_pos = nnz(s>0);
if num_pos>0
  Y1 = U(:,1:num_pos)*diag(s(1:num_pos))*V(:,1:num_pos)';
else
  Y1 = zeros(size(Y));
end
end

function Y1 = threshold_l1(Tau,Y)
% do thresholding on Y with Tau
% Inputs:
%   Tau: a positive scalar or a matrix with the same size as Y;
%   Y: a matrix;
% Outputs:
%   Y1: the matrix after thresholding on Y;

% S(tau,t) = t-tau, if t > tau;
%            t+tau, if t < -tau;
%            0,     otherwise;
% operate S() on each element of Y, with a commen tau, if the input Tau is
% a scalar, or with Tau_{ij} on Y_ij, if the input Tau is matrix;

Y1 = sign(Y).*max((abs(Y)-Tau),0);

end