function [adj_mat id_cell nNode flag ]= importGraph(fileName,titleLineNum,connection_type)
% read a graph from a text file

% Inputs:
%   fileName: text file name, the file should have the format
%       IdentifierOfNode_A   weight   IdentifierOfNode_B;
%      for instance: 
%       ACT1	pp	CCT4
%       ALG7	pp	HMG1
%       ALG7	pp	PHO88
%       ......

%   titleLineNum: Optional argument, the number of title lines, the first 
%       TITLELINENUM lines will be ignored when read the data, default 0;
%   conncection_type: Optional argument, a string indicating type of the weight; 
%       '%s': string
%       '%f': float number
%       '%d': integer number, ...
% Outputs:
%   adj_mat: a sparse square adjacent matrix of the graph;
%   id_cell: a column cell array with the same lenth of NODENUM, with
%     id_cell{i} the identifier of i-th node;
%   nNode: number of the nodes
%   flag: 1: succeed to produce the adjacent graph;  0: failed;

adj_mat = [];
id_cell = [];
nNode = 0;
if nargin<=1
    titleLineNum = 0;
end
if nargin<=2
    connection_type = '%s';
end

flag_weight = 0; % whether need to read weights of the edges 
switch lower(connection_type)
    case {'%s','%c','%x'}
        connection_type = '%*s'; % if the weights are character, the weight strings are neglected;
    otherwise
        flag_weight = 1;
end


fid = fopen(fileName,'r');
if fid == -1
    flag = 0;
    return;
end

% 1. read the data from the file

% ignore the first TITLELINENUM lines
if titleLineNum >0
    textscan(fid,'%*[^\n]',titleLineNum);
end
 
conversion_char = sprintf('%%s %s %%s %%*[^\\n]',connection_type); %   '%*[^\n]' : ignore the rest columns of the line

C = textscan(fid,conversion_char);

% 2. produce the correspondence of identifiers and their orders

if flag_weight
    ib  = 3; % ib: the column of inteactor B;
else
    ib = 2;
end
C{1} = strtrim(C{1});
C{ib} = strtrim(C{ib});
id_cell = unique(union(unique(C{1}),unique(C{ib}))); % identifiers of the nodes;
nNode = length(id_cell);

c1 = [ id_cell; C{1}];
c2 = [ id_cell; C{ib}];
len_c1 = length(c1);
len_edge = length(C{1}); % number of edges

[id_cell_1 m1_v n1_v] = unique(c1); % id_cell_1 = c1(m1_v); c1 = id_cell_1(n1_v);
[id_cell_2 m2_v n2_v] = unique(c2);

% 3. generate the adjacent matrix

if flag_weight
    adj_mat = sparse(n1_v(nNode+1: len_c1), n2_v(nNode+1:len_c1),C{2},nNode,nNode);
else
    adj_mat = sparse(n1_v(nNode+1: len_c1), n2_v(nNode+1:len_c1),ones(len_edge,1),nNode,nNode);
end
    

fclose(fid);
flag = 1;
end